<?php
/**
 * A generic class for handling users
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright(c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class User extends BaseFrameworkSystem implements ManageableUser, Registerable {
	/**
	 * Username of current user
	 */
	private $userName = "";

	/**
	 * Email of current user
	 */
	private $email = "";

	// Exceptions
	const EXCEPTION_USERNAME_NOT_FOUND   = 0xd00;
	const EXCEPTION_USER_EMAIL_NOT_FOUND = 0xd01;
	const EXCEPTION_USER_PASS_MISMATCH   = 0xd02;

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct ($class = "") {
		// Is the class name empty? Then this is not a specialized user class
		if (empty($class)) $class = __CLASS__;

		// Call parent constructor
		parent::__construct($class);

		// Set part description
		$this->setObjectDescription("Generic user class");

		// Create unique ID number
		$this->generateUniqueId();

		// Clean up a little
		$this->removeNumberFormaters();
		$this->removeSystemArray();
	}

	/**
	 * Creates an instance of this user class by a provided username. This
	 * factory method will check if the username is already taken and if not
	 * so it will throw an exception.
	 *
	 * @param	$userName		Username we need a class instance for
	 * @return	$userInstance	An instance of this user class
	 * @throws	UsernameMissingException	If the username does not exist
	 */
	public final static function createUserByUsername ($userName) {
		// Get a new instance
		$userInstance = new User();

		// Set the username
		$userInstance->setUserName($userName);

		// Check if the username exists
		if (!$userInstance->ifUsernameExists()) {
			// Throw an exception here
			throw new UsernameMissingException(array($userInstance, $userName), self::EXCEPTION_USERNAME_NOT_FOUND);
		}

		// Return the instance
		return $userInstance;
	}

	/**
	 * Creates an instance of this user class by a provided email address. This
	 * factory method will not check if the email address is there.
	 *
	 * @param	$email			Email address of the user
	 * @return	$userInstance	An instance of this user class
	 */
	public final static function createUserByEmail ($email) {
		// Get a new instance
		$userInstance = new User();

		// Set the username
		$userInstance->setEmail($email);

		// Return the instance
		return $userInstance;
	}

	/**
	 * Setter for username
	 *
	 * @param	$userName	The username to set
	 * @return	void
	 */
	public final function setUserName ($userName) {
		$this->userName = $userName;
	}

	/**
	 * Setter for email
	 *
	 * @param	$email	The email to set
	 * @return	void
	 */
	protected final function setEmail ($email) {
		$this->email = $email;
	}

	/**
	 * Getter for username
	 *
	 * @return	$userName	The username to get
	 */
	public final function getUsername () {
		return $this->userName;
	}

	/**
	 * Getter for email
	 *
	 * @return	$email	The email to get
	 */
	public final function getEmail () {
		return $this->email;
	}

	/**
	 * Determines wether the username exists or not
	 *
	 * @return	$exists		Wether the username exists
	 */
	public function ifUsernameExists () {
		// By default the username does not exist
		$exists = false;

		// Get a UserDatabaseWrapper instance
		$wrapperInstance = ObjectFactory::createObjectByConfiguredName('user_db_wrapper');

		// Create a search criteria
		$criteriaInstance = ObjectFactory::createObjectByConfiguredName('search_criteria');

		// Add the username as a criteria and set limit to one entry
		$criteriaInstance->addCriteria(UserDatabaseWrapper::DB_COLUMN_USERNAME, $this->getUsername());
		$criteriaInstance->setLimit(1);

		// Get a search result
		$result = $wrapperInstance->doSelectByCriteria($criteriaInstance);

		// Search for it
		if ($result->next()) {
			// Entry found
			$exists = true;
		} // END - if

		// Return the status
		return $exists;
	}

	/**
	 * Determines wether the email exists or not
	 *
	 * @return	$exists		Wether the email exists
	 */
	public function ifEmailAddressExists () {
		// By default the email does not exist
		$exists = false;

		// Get a UserDatabaseWrapper instance
		$wrapperInstance = ObjectFactory::createObjectByConfiguredName('user_db_wrapper');

		// Create a search criteria
		$criteriaInstance = ObjectFactory::createObjectByConfiguredName('search_criteria');

		// Add the username as a criteria and set limit to one entry
		$criteriaInstance->addCriteria(UserDatabaseWrapper::DB_COLUMN_EMAIL, $this->getEmail());
		$criteriaInstance->setLimit(1);

		// Get a search result
		$result = $wrapperInstance->doSelectByCriteria($criteriaInstance);

		// Search for it
		if ($result->next()) {
			// Entry found
			$exists = true;
		} // END - if

		// Return the status
		return $exists;
	}

	/**
	 * Checks if the supplied password hash in request matches with the stored
	 * in database.
	 *
	 * @param	$requestInstance	A requestable class instance
	 * @return	$matches			Wether the supplied password hash matches
	 */
	public function ifPasswordHashMatches (Requestable $requestInstance) {
		// By default nothing matches... ;)
		$matches = false;

		// Get a UserDatabaseWrapper instance
		$wrapperInstance = ObjectFactory::createObjectByConfiguredName('user_db_wrapper');

		// Create a search criteria
		$criteriaInstance = ObjectFactory::createObjectByConfiguredName('search_criteria');

		// Add the username as a criteria and set limit to one entry
		$criteriaInstance->addCriteria(UserDatabaseWrapper::DB_COLUMN_USERNAME, $this->getUserName());
		$criteriaInstance->setLimit(1);

		// Get a search result
		$result = $wrapperInstance->doSelectByCriteria($criteriaInstance);

		// Search for it
		if ($result->next()) {
			// Get the current entry (can only be one!)
			$entry = $result->current();

			// So does the hashes match?
			$matches = ($requestInstance->getRequestElement('pass_hash') === $entry['pass_hash']);
		} // END - if

		// Return the status
		return $matches;
	}

	/**
	 * Adds data for later complete update
	 *
	 * @param	$column		Column we want to update
	 * @param	$value		New value to store in database
	 * @return	void
	 */
	public function addUpdateData ($column, $value) {
		$this->partialStub("Column={$column}, value={$value}");
	}
}

// [EOF]
?>
