<?php
/**
 * A class for the configuration stuff implemented in a singleton design paddern
 *
 * NOTE: We cannot put this in inc/classes/ because it would be loaded (again)
 * in the class loader. See inc/loader/class_ClassLoader.php for instance
 *
 * @see			ClassLoader
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright(c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class FrameworkConfiguration implements Registerable {
	/**
	 * The framework's main configuration array which will be initialized with
	 * hard-coded configuration data and might be overwritten/extended by
	 * config data from the database.
	 */
	private $config = array();

	/**
	 * The configuration instance itself
	 */
	private static $cfgInstance = null;

	// Some constants for the configuration system
	const EXCEPTION_CONFIG_ENTRY_IS_EMPTY      = 0xc00;
	const EXCEPTION_CONFIG_ENTRY_WAS_NOT_FOUND = 0xc01;

	/**
	 * Protected constructor
	 */
	protected function __construct () {
		// Empty for now
	}

	/**
	 * "Create" a configuration instance
	 */
	public final static function createFrameworkConfiguration ($enableDebug = false) {
		/**
		 * For singleton design pattern because we only need a one-time-run
		 * through the initial configuration.
		 */
		if (is_null(self::$cfgInstance))  {
			// CFG: DEBUG-LEVEL
			@error_reporting(E_ALL | E_STRICT);

			/**
			 * Shall we enable the debug mode?
			 */
			if ($enableDebug) {
				define('DEBUG_MODE', true);
			}

			/**
			 * Crate a config instance
			 */
			self::$cfgInstance = new FrameworkConfiguration();
		}

		/**
		 * Return the instance
		 */
		return self::$cfgInstance;
	}

	/**
	 * Getter for an instance of this class
	 *
	 * @return	$cfgInstance	An instance of this class
	 */
	public final static function getInstance () {
		return self::$cfgInstance;
	}

	/**
	 * Setter for default time zone (must be correct!)
	 *
	 * @param		$zone	The time-zone string (e.g. Europe/Berlin)
	 * @return	void
	 */
	public final function setDefaultTimezone ($zone) {
		// At least 5.1.0 is required for this!
		if (version_compare(phpversion(), "5.1.0")) {
			@date_default_timezone_set($zone);
		}
	}

	/**
	 * Setter for runtime magic quotes
	 */
	public final function setMagicQuotesRuntime ($enableQuotes) {
		// Cast it to boolean
		$enableQuotes = (boolean) $enableQuotes;

		// Set it
		@set_magic_quotes_runtime($enableQuotes);
	}

	/**
	 * A private include loader
	 *
	 * @param	$arrayObject	The array object with all include files
	 * @return	void
	 */
	private function loadIncludes (ArrayObject $arrayObject) {
		// Load only if there are includes defined
		if (!is_null($arrayObject)) {
			for ($idx = $arrayObject->getIterator(); $idx->valid(); $idx->next()) {
				// Get include file
				$inc = $idx->current();

				// Is the file name really set?
				if (!empty($inc)) {
					// Base path added? (Uni* / Windows)
					if ((substr($inc, 0, 1) != "/") && (substr($inc, 1, 1) != ":")) {
						// Generate FQFN
						$fqfn = sprintf("%s/inc/extra/%s", PATH, $inc);
					} else {
						// Base path is already added
						$fqfn = $inc;
					}
				}

				// Include them all here
				require($fqfn);
			}
		}
	}

	/**
	 * Define the database type which must be valid and will not be verified.
	 *
	 * @param		$type	The database type. See path inc/database/.
	 * @return	void
	 */
	public function defineDatabaseType ($type) {
		// Is it defined or not?
		if (!defined('_DB_TYPE')) {
			// Cast to string
			$type = (string) $type;

			// Set the constant
			define('_DB_TYPE', $type);
		} else {
			// Already defined! But we cannot throw an exception here... :(
			ApplicationEntryPoint::app_die(sprintf("[%s:] Please define the database type only once in your application!",
				__CLASS__
			));
		}
	}

	/**
	 * Define the local file path
	 *
	 * @param		$path	The database type. See path inc/database/.
	 * @return	void
	 */
	public function definePath ($path) {
		// Cast to string
		$path = (string) $path;

		// Replace backslashes with slashes
		$path = str_replace("\\", "/", $path);

		// Is it defined or not?
		if ((!is_dir($path)) || (!is_readable($path))) {
			// Is not a valid path
			ApplicationEntryPoint::app_die(sprintf("[%s:] Invalid path (not found) specified. Please make sure it is created.",
				__CLASS__
			));
		} elseif (!defined('PATH')) {
			// Set the constant
			define('PATH', $path);
		} else {
			// Already defined! But we cannot throw an exception here... :(
			ApplicationEntryPoint::app_die(sprintf("[%s:] Please define the local file path only once in your application.",
				__CLASS__
			));
		}
	}

	/**
	 * Read a configuration element.
	 *
	 * @param		$cfgEntry	The configuration element
	 * @return	$cfgValue	The fetched configuration value
	 * @throws	ConfigEntryIsEmptyException	If $cfgEntry is empty
	 * @throws	ConfigEntryNotFoundException	If a configuration element
	 *									was not found
	 */
	public function readConfig ($cfgEntry) {
		// Cast to string
		$cfgEntry = (string) $cfgEntry;

		// Is a valid configuration entry provided?
		if (empty($cfgEntry)) {
			// Entry is empty
			throw new ConfigEntryIsEmptyException($this, self::EXCEPTION_CONFIG_ENTRY_IS_EMPTY);
		} elseif (!isset($this->config[$cfgEntry])) {
			// Entry was not found!
			throw new ConfigEntryNotFoundException(array(__CLASS__, $cfgEntry), self::EXCEPTION_CONFIG_ENTRY_WAS_NOT_FOUND);
		}

		// Debug message
		if ((defined('DEBUG_CONFIG')) || (defined('DEBUG_ALL'))) {
			echo "[".__METHOD__."] Configuration entry ".$cfgEntry." requested.<br />\n";
		}

		// Return the requested value
		return $this->config[$cfgEntry];
	}

	/**
	 * Set a configuration entry.
	 *
	 * @param		$cfgEntry	The configuration entry we want to add/change
	 * @param		$cfgValue	The configuration value we want to set
	 * @return	void
	 * @throws	ConfigEntryIsEmptyException	If $cfgEntry is empty
	 */
	public final function setConfigEntry ($cfgEntry, $cfgValue) {
		// Cast to string
		$cfgEntry = (string) $cfgEntry;
		$cfgValue = (string) $cfgValue;

		// Is a valid configuration entry provided?
		if (empty($cfgEntry)) {
			// Entry is empty
			throw new ConfigEntryIsEmptyException($this, self::EXCEPTION_CONFIG_ENTRY_IS_EMPTY);
		}

		// Set the configuration value
		$this->config[$cfgEntry] = $cfgValue;

		// Resort the array
		ksort($this->config);
	}

	/**
	 * Compatiblity method to return this class' name
	 *
	 * @return	__CLASS__		This class' name
	 */
	public function __toString () {
		return get_class($this);
	}
} // END - class

// [EOF]
?>
